extern crate time;

use std::fs::File;
use std::io::prelude::*;
use std::io;
use super::Sensor;

struct StatFiles {
    rx: File,
    tx: File
}

struct Stats {
    rx: i64,
    tx: i64
}

pub struct NetSpeedSensor {
    files: Vec<StatFiles>,
    stats: Option<Stats>,
    rate: Option<Stats>,
    last_time: i64
}

impl NetSpeedSensor {
    pub fn new(devices: &Vec<&str>) -> NetSpeedSensor {
        let files: Vec<StatFiles> = devices.iter()
            .flat_map(|dev| open_stats(&dev).ok())
            .collect();

        NetSpeedSensor {
            files: files,
            stats: None,
            rate: None,
            last_time: 0
        }
    }
}

impl Sensor for NetSpeedSensor {
    fn icon(&self) -> String {
        "NET".to_string()
    }

    fn status(&self) -> String {
        match self.rate.as_ref() {
            Some(rate) => format!("{}↓ {}↑", format_bytes(rate.rx), format_bytes(rate.tx)),
            None => "?".to_string()
        }
    }

    fn process(&mut self) {
        let curr_time = time::get_time().sec;

        let stats = self.files
            .iter_mut()
            .flat_map(|file| read_stats(file).ok())
            .fold(Stats { rx: 0, tx: 0 }, |acc, elem| Stats {
                rx: acc.rx + elem.rx,
                tx: acc.tx + elem.tx
            });

        let diff_time = curr_time - self.last_time;
        let output = match (self.stats.as_ref(), diff_time) {
            (_, 0) | (None, _) => None,
            (Some(pstats), diff_time) => {
                let rx = (stats.rx - pstats.rx) / diff_time;
                let tx = (stats.tx - pstats.tx) / diff_time;
                Some(Stats { rx: rx, tx: tx })
            }
        };

        self.last_time = curr_time;
        self.stats = Some(stats);
        self.rate = output;
    }
}

fn format_bytes(bytes: i64) -> String {
    let kib = bytes >> 10;
    if kib > 1024 {
        format!("{:.*} M", 1, (kib as f32) / 1024.0)
    }
    else {
        format!("{} K", kib)
    }
}

fn open_stats(device: &str) -> Result<StatFiles, io::Error> {
    let path = format!("/sys/class/net/{}/statistics", device);
    let rx_file = try!(File::open(format!("{}/rx_bytes", path)));
    let tx_file = try!(File::open(format!("{}/tx_bytes", path)));
    Ok(StatFiles {
        rx: rx_file,
        tx: tx_file
    })
}

fn read_stats(files: &mut StatFiles) -> Result<Stats, io::Error> {
    Ok(Stats {
        rx: read_bytes(&mut files.rx),
        tx: read_bytes(&mut files.tx)
    })
}

fn read_bytes(f: &mut File) -> i64 {
    let mut s = String::new();
    assert!(f.read_to_string(&mut s).is_ok());
    let i : i64 = s.trim().parse().unwrap();
    assert!(f.seek(io::SeekFrom::Start(0)).is_ok());
    i
}
